import click
from freshenv.cloud.config import view_config
from freshenv.cloud.fetch import fetch_environment
from freshenv.cloud.ls import list_environments
from freshenv.cloud.push import push_environment


@click.group(name="cloud")
def cloud() -> None:
    """Save and share your custom environments on the cloud."""


@cloud.command("ls")
@click.argument("plan", default="personal", type=click.Choice(["freshenv", "personal"]), metavar="plan")
def ls(plan: str) -> None:
    """List cloud environments."""
    list_environments(plan)

@cloud.command("push")
@click.argument("environment_name")
@click.argument("plan", default="personal",type=click.Choice(["freshenv", "personal"]),  metavar="plan")
def push(environment_name: str, plan: str) -> None:
    """Upload an environment to the cloud."""
    push_environment(environment_name, plan)

@cloud.command("fetch")
@click.argument("environment_name")
@click.argument("plan", default="personal",type=click.Choice(["freshenv", "personal"]),  metavar="plan")
def fetch(environment_name: str, plan: str) -> None:
    """Download an environment from the cloud."""
    fetch_environment(environment_name, plan)

@cloud.command("config")
@click.argument("plan", default="personal", type=click.Choice(["freshenv", "personal"]),  metavar="plan")
def config(plan: str) -> None:
    """View personal and freshenv cloud configurations."""
    view_config("cloud."+plan)



from docker import APIClient, errors
from rich import print
import boto3
from os import remove
from botocore import exceptions
from freshenv.cloud.config import get_config


def export_environment(environment_name: str) -> str:
    try:
        client = APIClient(base_url="unix://var/run/docker.sock")
        exported_container = client.export(container=environment_name)
        with open(environment_name + ".tar", "wb") as f:
            for chunk in exported_container:
                f.write(chunk)
        return f.name
    except errors.NotFound:
        print(" :man_shrugging: Environment not found. Please check if the environment exists.")
    except errors.APIError:
        print(":cold_sweat: Could not export environment. Please contact the developer or report an issue. :ticket:")
    except errors.DockerException:
        print(":cross_mark_button: Docker not installed or running. ")
    except Exception as e:
        print(f"Unknown exception: {e}")
    return ""


def remove_tar_file(file_name: str) -> None:
    remove(file_name)


def push_environment_to_aws(environment_name: str, config_obj: dict) -> None:
    try:
        session = boto3.session.Session(profile_name=config_obj["aws_profile"])
        s3_client = session.client('s3')
        s3_client.head_bucket(Bucket=config_obj["bucket"])
        print(":link: Uploading environment to the cloud.")
        file_name = export_environment(environment_name)
        if file_name == "":
            return
        s3_client.upload_file(file_name, config_obj["bucket"], file_name)
        print(":white_check_mark: Environment uploaded successfully.")
        remove_tar_file(file_name)
    except exceptions.ClientError as client_error:
        if client_error.response['Error']['Code'] == 'NoSuchBucket':
            print(":person_shrugging: Bucket does not exist.")
    except exceptions.ProfileNotFound:
        print(":person_shrugging: config profile does not exist.")


def push_environment(environment_name: str, plan: str) -> None:
    """Upload a custom environment to the cloud."""
    if plan == "personal":
        config_obj = get_config(f"cloud.{plan}")
        if not config_obj:
            return
        if config_obj and config_obj["provider"]:
            provider = config_obj["provider"]
            if provider == "aws":
                push_environment_to_aws(environment_name, config_obj)
            else:
                print(
                    f":building_construction:  {provider} provider not supported.")
        else:
            print(":person_facepalming: No provider configured.")
    else:
        print(":white_sun_with_small_cloud:  The freshenv cloud plan is coming soon. Please visit the website for more information.")



from configparser import ConfigParser
from os import path
from typing import Dict
from rich import pretty, print
from freshenv.build import config_exists, create_file, key_exists, get_key_values_from_config

homedir = path.expanduser("~")
freshenv_config_location = homedir + "/.freshenv/freshenv"


def check_run(config_type: str) -> bool:
    """Check if cloud configuration is valid."""
    if not config_exists():
        print(f":card_index: No config file found. Creating an empty config at {freshenv_config_location}.")
        create_file(freshenv_config_location)
        return False
    if not key_exists(config_type):
        print(f":exclamation_mark: A [bold]{config_type}[/bold] cloud has not been configured.")
        return False
    if not mandatory_keys_exists(config_type):
        print(f":exclamation_mark: missing mandatory keys in {config_type} configuration for cloud.")
        return False
    return True


def mandatory_keys_exists(config_type: str) -> bool:
    config = ConfigParser()
    config.read(freshenv_config_location)
    if "personal" in config_type:
        if "provider" not in config[config_type]:
            return False
        if "aws_profile" not in config[config_type]:
            return False
        if "bucket" not in config[config_type]:
            return False
    elif "freshenv" in config_type:
        if "apikey" not in config[config_type]:
            return False
    else:
        return False
    return True


def get_config(config_type: str) -> Dict[str, str]:
    if not check_run(config_type):
        return {}
    cloud_config = dict(get_key_values_from_config(config_type))
    return cloud_config


def view_config(config_type: str) -> None:
    """View personal and freshenv cloud configurations."""
    cloud_config = get_config(config_type)
    if not cloud_config:
        return
    pretty.pprint(cloud_config)



import boto3
from botocore import exceptions
from rich import print
from rich.tree import Tree
from freshenv.cloud.config import get_config


def list_environments_from_aws(config_obj: dict) -> None:
    """List your custom cloud environments from AWS."""
    try:
        session = boto3.session.Session(profile_name=config_obj["aws_profile"])
        s3_client = session.client('s3')
        bucket_objs = s3_client.list_objects_v2(Bucket=config_obj["bucket"])
        print(":link: Listing your cloud environments.")
        tree = Tree("Cloud Environments")
        if "Contents" in bucket_objs and len(bucket_objs["Contents"]) > 0:
            for obj in bucket_objs["Contents"]:
                tree.add(obj["Key"])
        else:
            tree.add("No cloud environments found.")
        print(tree)
    except exceptions.ClientError as client_error:
        if client_error.response['Error']['Code'] == 'NoSuchBucket':
            print(":person_shrugging: Bucket does not exist.")
    except exceptions.ProfileNotFound:
        print(":person_shrugging: config profile does not exist.")

def list_environments(plan: str) -> None:
    """List your custom cloud environments."""
    if plan == "personal":
        config_obj = get_config(f"cloud.{plan}")
        if not config_obj:
            return
        if config_obj and config_obj["provider"]:
            provider = config_obj["provider"]
            if provider == "aws":
                list_environments_from_aws(config_obj)
            else:
                print(f":building_construction: {provider} provider not supported.")
        else:
            print(":person_facepalming: No provider configured.")
    else:
        print(":white_sun_with_small_cloud:  The freshenv cloud plan is coming soon. Please visit the website for more information.")



from docker import APIClient, errors
from botocore import exceptions
import boto3
from os import remove
from freshenv.cloud.config import get_config
from freshenv.start import start


def fetch_environment_from_aws(environment_name: str, config_obj: dict) -> None:
    try:
        session = boto3.session.Session(profile_name=config_obj["aws_profile"])
        s3_client = session.client('s3')
        s3_client.head_bucket(Bucket=config_obj["bucket"])
        print(":link: Downloading environment from the cloud.")
        s3_client.download_file(config_obj["bucket"], environment_name + ".tar", environment_name + ".tar")
        print(":white_check_mark: Environment downloaded successfully.")
    except exceptions.ClientError as client_error:
        if client_error.response['Error']['Code'] == 'NoSuchBucket':
            print(":person_shrugging: Bucket does not exist.")
    except exceptions.ProfileNotFound:
        print(":person_shrugging: config profile does not exist.")
    except Exception as e:
        print(f"Unknown exception: {e}")


def import_container(environment_name: str) -> None:
    try:
        client = APIClient(base_url="unix://var/run/docker.sock")
        client.import_image(environment_name + ".tar", environment_name)
        print(":white_check_mark: Environment imported successfully.")
        remove(environment_name + ".tar")
    except errors.APIError:
        print(":cold_sweat: Could not import environment. Please contact the developer or report an issue. :ticket:")
    except errors.DockerException:
        print(":cross_mark_button: Docker not installed or running. ")
    except Exception as e:
        print(f"Unknown exception: {e}")

def fetch_environment(environment_name: str, plan: str) -> None:
    """Fetch a custom environment from the cloud."""
    if plan == "personal":
        config_obj = get_config(f"cloud.{plan}")
        if not config_obj:
            return
        if config_obj and config_obj["provider"]:
            provider = config_obj["provider"]
            if provider == "aws":
                fetch_environment_from_aws(environment_name, config_obj)
                import_container(environment_name)
                start(environment_name)
            else:
                print(f":building_construction:  {provider} provider not supported.")
        else:
            print(":person_facepalming: No provider configured.")
    else:
        print(":white_sun_with_small_cloud:  The freshenv cloud plan is coming soon. Please visit the website for more information.")
